`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    00:21:14 11/07/2012 
// Design Name: 
// Module Name:    LineGen 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module LineGen( input clk108,
					 input countflag,				//flag, plot inside borders
					 input vblank,
					 input vsync,
					 output reg [15:0] X = 0,			//H component of pixel to BRAM
					 output reg [15:0] Y = 0,			//V component of pixel to BRAM
					 output reg WE = 0						// write enable for BRAM
					);
					
reg [15:0] x0;
reg [15:0] y0;
reg [15:0] x1;	
reg [15:0] y1;
reg [15:0] dx;
reg [15:0] dy;
reg [15:0] num;
reg [0:0] sx;

reg [2:0] state;
parameter init_state = 0, prep_state = 1, Hplot_state = 2, Vplot_state = 3;

always @(posedge clk108)
if ( vblank & vsync ) 
	state <= init_state;
	else
		case (state)
			init_state :
					state <= prep_state;
			prep_state :
				if ( countflag )
					if ( dx > dy )
						state <= Hplot_state;
					else
						state <= Vplot_state;
			Hplot_state :
				if ( !countflag )
				state <= init_state;
			Vplot_state :
				if ( !countflag )
				state <= init_state;
		endcase
				
always @(posedge clk108) begin
case (state)
init_state:
	begin
		x0 <= 10;
		y0 <= 10;
		x1 <= 200;
		y1 <= 30;
		dy <= y1 - y0;						// CPU sending coordinates must follow y1>y0
			if ( x0 > x1 ) begin			
				dx <= x0 - x1;
				sx <= 0;						// negative "/" slope, sx=0
			end
				else begin					
					dx <= x1 - x0;
					sx <= 1;					// positive "\" slope, sx=1
				end
	end

prep_state:
		if ( dx > dy ) 
			num <= dy >> 1;
			else
				num <= dx >> 1;
		
Hplot_state:								// dx/dy=(x1-x0)/(y1-y0).  Hplot_state when dx is larger than dy
		if ( y0 < y1 ) begin
			X <= x0;							// output pixel coordinates to BRAM
			Y <= y0;
			num <= num + dy;				// keep plotting on same raster line...
			x0 <= sx ? x0 + 1 :
						  x0 - 1;
				if ( num >= dx ) begin	// ...until (dx/dy) > 1
					num <= dy >> 1;		// reset num
					y0 <= y0 + 1;			// next raster line
				end
		end
		else begin							// at end of line (y0=y1) reset origin
			x0 <= 10;
			y0 <= 10;
		end
	
Vplot_state:
		if ( x0 < x1 ) begin
			Y <= y0;
			X <= x0;		
			num <= num + dx;
			y0 <= y0 + 1;
				if ( num >= dy ) begin
					num <= dx;                                                                                                                            ;
					x0 <= sx ? x0 + 1 :
								  x0 - 1;
				end
		end
		else begin
			x0 <= 10;
			y0 <= 10;
		end
endcase
end

endmodule
