module VDACif( input clk108,
					input hstart,					
					input hblank,
					input vblank,
					input vsync,
					input [15:0] X1,			//horizontal component of line data
					input [15:0] Y1,
					output reg [9:0] Z = 0,							//diagonal counter
					output reg [4:0] Red_data = 0,
					output reg [5:0] Green_data = 0,
					output reg [4:0] Blue_data = 0,
					output reg DACBLANKn = 1,
					output reg countflag
					);

reg [15:0] Xcount = 0;							//horizontal counter
reg [15:0] Ycount = 0;							//vertical counter

parameter
Xmax = 640,
Ymax = 480,
Zmax = 800;									//diagonal max

always @(posedge clk108)
      if ( hstart )
         countflag <= 1;				//countflag active in display area
      else if ( hblank )
         countflag <= 0;
			
always @(posedge clk108)
	if ( countflag )
		Xcount <= Xcount + 1'b1;							//count horizontal position inside the border
		else 
			Xcount <= 0;

always @(posedge clk108)				//
	 if ( vblank )
			Ycount <= 0;
		else if ( Xcount == Xmax )		
			Ycount <= Ycount + 1'b1;
						
always @(posedge clk108)
	if ( countflag )
		if ( Z == Zmax - 1'b1 )
			Z <= 0;
			else 
				Z <= Z + 1'b1;
				
always @(posedge clk108)								//outgoing data to videoDAC
	if ( Ycount == Y1 | Xcount == X1 ) begin
		Red_data <= 0;
		Green_data <= 6'b111111;
		Blue_data <= 0;								//Priority 1, plot green pixel
	end
	
		else if ( !countflag ) begin
			Red_data <= 0;
			Green_data <= 0;
			Blue_data <= 0;						//Priority 2, blue border
		end
		else if ( vblank & vsync ) begin			
			Red_data <= 0;
			Green_data <= 0;
			Blue_data <= 0;								//send black during vertical retrace
		end
		
	else begin
		Red_data <= 0;
		Green_data <= 0;
		Blue_data <= 0;							//Priority 4, even pixels blue
	end
	
endmodule



